/* Primitive "Paint" program --- paints up to MAXOBJECTS red triangles. */

#include "Angel.h"

/* Width and height of display window */
const int w = 512;
const int h = 512;

/* Basic color data */
int currentColor = 0;
const vec3  base_colors[] = {
   vec3( 1.0, 0.0, 0.0 ), // red
   vec3( 0.0, 1.0, 0.0 ), // green
   vec3( 0.0, 0.0, 1.0 ), // blue
   vec3( 0.0, 0.0, 0.0 )  // black
};

/* Max vertices per object */
const int MAXVERTICES = 10;

/* points and colors are used to collect vertex data while we're "building"
 * an object
 */
vec2 points[MAXVERTICES];
vec3 colors[MAXVERTICES];

/* Next available location in points and colors */
int  Index = 0;

/* Maximum number of "paintable" objects */
const int MAXOBJECTS = 10;

/* Data structure for storing an object --- the vao and the number of
 * stored in the vao.
 */
typedef struct object{
   GLuint vao;
   GLuint numVertices;
} object;

/* Our array of objects */
object objects[MAXOBJECTS];

/* Index of next available object */
int nextObject = 0;

/* Identifier for the shader programs */
GLuint program;

//----------------------------------------------------------------------------

/* Create and return a vao using the current vertex information in points
 * and colors, and the current (unchanging) shader programs.
 */

GLuint
createVao( void )
{
   GLuint vao;
   GLuint buffer;

   // Create a vertex array object
   glGenVertexArrays( 1, &vao );
   glBindVertexArray( vao );

   // Create and initialize a buffer object
   glGenBuffers( 1, &buffer );
   glBindBuffer( GL_ARRAY_BUFFER, buffer );

   // First, we create an empty buffer of the size we need by passing
   //   a NULL pointer for the data values
   glBufferData( GL_ARRAY_BUFFER, sizeof(points) + sizeof(colors),
                 NULL, GL_STATIC_DRAW );

   // Next, we load the real data in parts.  We need to specify the
   //   correct byte offset for placing the color data after the point
   //   data in the buffer.  Conveniently, the byte offset we need is
   //   the same as the size (in bytes) of the points array, which is
   //   returned from "sizeof(points)".
   glBufferSubData( GL_ARRAY_BUFFER, 0, sizeof(points), points );
   glBufferSubData( GL_ARRAY_BUFFER, sizeof(points), sizeof(colors), colors );

   glUseProgram( program );

   // Initialize the vertex position attribute from the vertex shader    
   GLuint vPosition = glGetAttribLocation( program, "vPosition" );
   glEnableVertexAttribArray( vPosition );
   glVertexAttribPointer( vPosition, 2, GL_FLOAT, GL_FALSE, 0,
                          BUFFER_OFFSET(0) );

   // Likewise, initialize the vertex color attribute.  Once again, we
   //    need to specify the starting offset (in bytes) for the color
   //    data.  Just like loading the array, we use "sizeof(points)"
   //    to determine the correct value.
   GLuint vColor = glGetAttribLocation( program, "vColor" );
   glEnableVertexAttribArray( vColor );
   glVertexAttribPointer( vColor, 3, GL_FLOAT, GL_FALSE, 0,
                          BUFFER_OFFSET(sizeof(points)) );

   return vao;
}
//----------------------------------------------------------------------------

void
init( void )
{
   // Load shaders and use the resulting shader program
   program = InitShader( "vshader24.glsl", "fshader24.glsl" );

   glClearColor( 1.0, 1.0, 1.0, 1.0 ); /* white background */
}

//----------------------------------------------------------------------------

/* Paint the current objects */

void
display( void )
{
   glClear( GL_COLOR_BUFFER_BIT );

   for (int i = 0; i < nextObject; i++)
   {
      glBindVertexArray( objects[i].vao );
      glDrawArrays( GL_TRIANGLES, 0, objects[i].numVertices );
   }

   glFlush();
}

//----------------------------------------------------------------------------

void
keyboard( unsigned char key, int x, int y )
{
   switch ( key ) {
   case 033:
      exit( EXIT_SUCCESS );
      break;
   }
}

//----------------------------------------------------------------------------

/* Convert window coordinates, in pixels, back to world coordinates */

vec2 convert(int x, int y)
{
   return vec2((GLfloat) x / (GLfloat) w,
               (GLfloat) y / (GLfloat) h);
}

//----------------------------------------------------------------------------

/* Mouse callback function */

void
mouse( int button, int state, int x, int y )
{
   if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
   {
      std::cout << x << ", " << y << std::endl;
      points[Index] = convert(x, y);
      std::cout << points[Index].x << ", " << points[Index].y << std::endl;
      colors[Index++] = base_colors[currentColor];

      if (Index == 3 && nextObject < MAXOBJECTS)
      {
         objects[nextObject].numVertices = Index;
         Index = 0;
         objects[nextObject++].vao = createVao();
         glutPostRedisplay();
         
      }
   }
}

//----------------------------------------------------------------------------

int
main( int argc, char **argv )
{
   glutInit( &argc, argv );
   glutInitDisplayMode( GLUT_RGBA );
   glutInitWindowSize( w, h );
   glutInitContextVersion( 3, 2 );
   glutInitContextProfile( GLUT_CORE_PROFILE );
   glutCreateWindow( "Simple Paint Program" );

   glewExperimental = GL_TRUE;
   glewInit();
   
   init();

   glutDisplayFunc( display );
   glutKeyboardFunc( keyboard );
   glutMouseFunc( mouse );

   glutMainLoop();
   return 0;
}
